/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package mentionmatch;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author gbl
 */
class Comparison {
    
    List<MatrixComparison> parts;

    Comparison(ListOfMentions measure, ListOfMentions reference) {
        parts=new ArrayList<>();

        // Get the first and second mentions that have refOffset set
        // (i.e. splice begins) from both mention lists
        Mention measureFirst = measure.getMentions().get(measure.firstRefOffset);
        Mention refFirst = reference.getMentions().get(reference.firstRefOffset);
        
        Mention measureLast = measure.getMentions().get(measure.secondRefOffset);
        Mention refLast = reference.getMentions().get(reference.secondRefOffset);

        // and do some sanity checks
        
        if (measureFirst.refIndex != 0 || refFirst.refIndex != 0
        ||  measureLast.refIndex != 0 || refLast.refIndex != 0
        ||  measureFirst.anaFixed != 1 || refFirst.anaFixed != 1
        ||  measureLast.anaFixed != 1 || refLast.anaFixed != 1) {
            System.err.println("Splice references don't match up");
            System.exit(1);
        }
        
        // Divide the whole list into parts that have matching first and last
        // fixed points (anaFixed equal). For example, this list
        //
        // A 0          P 0
        // B 1          Q -1
        // C 3          R -1
        // D 0          S 1
        //              T -1
        //              U 2
        //              V -1
        //              W 3
        //              X 0
        // will have [AB] compared to [PS], [BC] to [SW] and [CD] to [WX].
        // Note the missing anaFixed==2 in the left table which means we
        // need to skip U on the right.
        
        int measureStart=measure.firstRefOffset;            // 'A' in comment
        int referenceStart=reference.firstRefOffset;        // 'P' in comment
        int measureEnd, referenceEnd;
        for (;;) {
            measureEnd=findNextFixedPoint(measure, measureStart);           // find 'B' from 'A'
            referenceEnd=findNextFixedPoint(reference, referenceStart);     // find 'S' from 'P'
            
            // Handle the "missing" case from the example; the 'End's are at
            // C/U here. The one that has the lower anaFixed (but not 0) needs
            // advance to the next fixed point.
            
            while (measure.getMentionAt(measureEnd).refIndex != reference.getMentionAt(referenceEnd).refIndex) {
                if (measure.getMentionAt(measureEnd).refIndex > 0 && 
                    measure.getMentionAt(measureEnd).refIndex < reference.getMentionAt(referenceEnd).refIndex)
                    measureEnd=findNextFixedPoint(measure, measureEnd);
                else if (reference.getMentionAt(referenceEnd).refIndex > 0 && 
                    measure.getMentionAt(measureEnd).refIndex > reference.getMentionAt(referenceEnd).refIndex)
                    referenceEnd=findNextFixedPoint(reference, referenceEnd);
                else {
                    System.err.println("This should not happen; bug when finding next matching references");
                    System.err.printf("starts = %d <> %d, ends = %d <> %d\n", measureStart, referenceStart, measureEnd, referenceEnd);
                    System.exit(1);
                }
            }
            
            parts.add(new MatrixComparison(measure, measureStart, measureEnd, reference, referenceStart, referenceEnd));
            if (measure.getMentionAt(measureEnd).refIndex == 0 && reference.getMentionAt(referenceEnd).refIndex == 0)
                break;
            measureStart=measureEnd;
            referenceStart=referenceEnd;
        }
    }
    
    /**
     * Find the next point, in a mentions list, that has anaFixed set, beginning from startIndex.
     * @param list the list of mentions to be searched
     * @param startIndex starting point
     * @return the first index > startIndex that has a mention with anaFixed != 0
     */
    private int findNextFixedPoint(ListOfMentions list, int startIndex) {
        int endIndex=startIndex+1;
        while (list.getMentionAt(endIndex).anaFixed == 0) {
            endIndex++;
        }
        return endIndex;
    }

    void process() {
        for (MatrixComparison part: parts) {
            part.process();
        }
    }

    void print(PrintStream out) {
        for (MatrixComparison part: parts) {
            part.print(out);
        }
    }
}
